"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var api_1 = require("../../api");
var getFps = function (fps) { return Number(api_1.isRef(fps) ? fps.value : fps); };
var calcFpsInterval = function (fps) { return 1000 / fps; };
var fpsLimit = 120;
function useRafFn(callback, 
// Note: a value greater than 120 will disable the fps check logic
// giving maximum precision and smoothness
fps, runOnMount) {
    if (fps === void 0) { fps = fpsLimit + 1; }
    if (runOnMount === void 0) { runOnMount = true; }
    var isRunning = api_1.ref(false);
    // Using a computed here allows us to update fps
    // dynamically from user's input
    var fpsInterval = api_1.computed(function () { return calcFpsInterval(getFps(fps)); });
    var isPausedGuard = false;
    var startTime = 0;
    var timeNow = 0;
    var timeWhenPaused = 0;
    var timeDelta = 0;
    var loop = function (timeStamp) {
        if (!startTime)
            startTime = timeStamp;
        if (!isRunning.value)
            return;
        if (isPausedGuard) {
            // Save the time when we pause the loop so that later we can
            // adjust the time we return to the callback
            timeWhenPaused = timeStamp - startTime - timeNow;
            isPausedGuard = false;
        }
        // Adjust timeNow to account for startTime and timeWhenPaused
        timeNow = timeStamp - startTime - timeWhenPaused;
        // Always run the callback if fps is greater than fpsLimit
        var callbackShouldAlwaysRun = getFps(fps) > fpsLimit;
        if (callbackShouldAlwaysRun) {
            // Store timeDelta for future computations
            timeDelta = timeNow;
            callback(timeNow);
        }
        // Run callback only when !callbackShouldAlwaysRun
        // and the given fps matches the lapsed time
        if (!callbackShouldAlwaysRun) {
            var elapsedTime = Math.ceil(timeNow - timeDelta);
            if (elapsedTime > fpsInterval.value) {
                // Store timeDelta for future computations
                timeDelta = timeNow;
                callback(timeNow);
            }
        }
        // Run loop again recursively
        requestAnimationFrame(loop);
    };
    var start = function () {
        isRunning.value = true;
        requestAnimationFrame(loop);
    };
    var stop = function () {
        isRunning.value = false;
        isPausedGuard = true;
    };
    api_1.onMounted(function () { return runOnMount && start(); });
    api_1.onUnmounted(stop);
    return {
        isRunning: isRunning,
        start: start,
        stop: stop
    };
}
exports.useRafFn = useRafFn;
